@@ -197,6 +197,8 @@ static void keyboard_key(void *data, struct wl_keyboard *keyboard,
 
   push_keypress(window, key);
 
+  wl_display_roundtrip(window->wl_display);
+
   if (imv_keyboard_should_key_repeat(window->keyboard, key)) {
     /* Kick off the key-repeat timer for the current key */
     window->repeat_scancode = key;
@@ -543,6 +545,7 @@ static void update_scale(struct imv_window *window)
     window->scale = new_scale;
     wl_surface_set_buffer_scale(window->wl_surface, window->scale);
     wl_surface_commit(window->wl_surface);
+    wl_display_roundtrip(window->wl_display);
     size_t buffer_width = window->width * window->scale;
     size_t buffer_height = window->height * window->scale;
     wl_egl_window_resize(window->egl_window, buffer_width, buffer_height, 0, 0);
@@ -555,7 +558,8 @@ static void update_scale(struct imv_window *window)
           .width = window->width,
           .height = window->height,
           .buffer_width = buffer_width,
-          .buffer_height = buffer_height
+          .buffer_height = buffer_height,
+          .scale = window->scale,
         }
       }
     };
@@ -634,8 +638,9 @@ static void toplevel_configure(void *data, struct xdg_toplevel *toplevel,
       .resize = {
         .width = window->width,
         .height = window->height,
-        .buffer_width = window->width * window->scale,
-        .buffer_height = window->height * window->scale
+        .buffer_width = buffer_width,
+        .buffer_height = buffer_height,
+        .scale = window->scale,
       }
     }
   };
